from django.db import models
from django.contrib.auth.models import User
class Session(models.Model):
    session = models.CharField(max_length=20, unique=True, blank=True, null=True)
    seal = models.ImageField(upload_to='seal_img/%Y/%m/%d', null=True, blank=True, verbose_name="University's Seal")
    current = models.BooleanField(max_length=5, default = False)
    reg_deadline = models.DateTimeField(null=True, blank=True)
    created = models.DateTimeField(auto_now_add=True, null=True)
    updated = models.DateTimeField(auto_now=True, null=True)

    class Meta:
        ordering = ('-created',)

    def __str__(self):
        return str(self.session)

class Semester(models.Model):
    session = models.ForeignKey('management.Session', on_delete=models.SET_NULL, null=True)
    semester = models.CharField(max_length=20, null=True)
    current = models.BooleanField(max_length=5, default = False)
    course_recorded = models.ManyToManyField('management.course', blank=True, related_name="course_semester")
    hod_approval = models.ManyToManyField('management.department', blank=True, related_name="semester_dept")
    dean_approval = models.ManyToManyField('management.faculty', blank=True, related_name="semester_dean")
    vc_approval = models.BooleanField(max_length=5, default = False)
    created = models.DateTimeField(auto_now_add=True, null=True)
    updated = models.DateTimeField(auto_now=True, null=True)

    def __str__(self):
        try:
            return str(self.semester) + " - " + str(self.session)
        except:
            return str(self.id)

    @property
    def imageURL(self):
        try:
            url = self.image.url
        except:
            url = ''
        return url

    class Meta:
        ordering = ('-created',)

class Level(models.Model):
    level = models.IntegerField(blank=True, unique=True, default = 0)
    # teacher = models.ForeignKey('users.Person', on_delete=models.SET_NULL, null=True)
    # signature = models.ImageField(upload_to='signature_img/%Y/%m/%d', null=True, blank=True, verbose_name="Class Teacher's Signature")
    created = models.DateTimeField(auto_now_add=True, null=True)
    updated = models.DateTimeField(auto_now=True, null=True)

    def __str__(self):
        try:
            return str(self.level)
        except:
            return str(self.id)

    @property
    def imageURL(self):
        try:
            url = self.image.url
        except:
            url = ''
        return url

    class Meta:
        ordering = ('-created',)
        # verbose_name = 'Class'
        # verbose_name_plural = 'Classes'

class Faculty(models.Model):
    faculty = models.CharField(max_length=50, null=True, unique=True)
    dean = models.ImageField(upload_to='dean/%Y/%m/%d', null=True, blank=True, verbose_name="Dean's Signature")
    the_dean = models.ForeignKey('users.Person', on_delete=models.SET_NULL, null=True, related_name="dean_faculty", verbose_name="Dean")
    # teacher = models.ForeignKey('users.Person', on_delete=models.SET_NULL, null=True)
    # signature = models.ImageField(upload_to='signature_img/%Y/%m/%d', null=True, blank=True, verbose_name="Class Teacher's Signature")
    created = models.DateTimeField(auto_now_add=True, null=True)
    updated = models.DateTimeField(auto_now=True, null=True)

    def __str__(self):
        try:
            return str(self.faculty)
        except:
            return str(self.id)

    class Meta:
        ordering = ('-created',)
        # verbose_name = 'Class'
        verbose_name_plural = 'Faculties'

class Department(models.Model):
    department = models.CharField(max_length=50, null=True, unique=True)
    faculty = models.ForeignKey('management.Faculty', on_delete=models.SET_NULL, null=True)
    hod = models.ForeignKey('users.Person', on_delete=models.SET_NULL, null=True, related_name="hod_dept", verbose_name="HoD")
    adviser = models.ImageField(upload_to='adviser/%Y/%m/%d', null=True, blank=True, verbose_name="HoD's Signature")
    # teacher = models.ForeignKey('users.Person', on_delete=models.SET_NULL, null=True)
    # signature = models.ImageField(upload_to='signature_img/%Y/%m/%d', null=True, blank=True, verbose_name="Class Teacher's Signature")
    created = models.DateTimeField(auto_now_add=True, null=True)
    updated = models.DateTimeField(auto_now=True, null=True)

    def __str__(self):
        try:
            return str(self.department)
        except:
            return str(self.id)

    class Meta:
        ordering = ('-created',)
        # verbose_name = 'Class'
        # verbose_name_plural = 'Faculties'

class Programme(models.Model):
    programme = models.CharField(max_length=50, null=True, unique=True)
    amount = models.DecimalField(max_digits=9, decimal_places=2, blank=True, null=True)
    amount_nc = models.DecimalField(max_digits=9, decimal_places=2, blank=True, null=True)
    apf = models.DecimalField(max_digits=9, decimal_places=2, blank=True, null=True)
    mind = models.DecimalField(max_digits=9, decimal_places=2, blank=True, null=True)
    department = models.ForeignKey('management.Department', on_delete=models.SET_NULL, null=True)
    advisor = models.ForeignKey('users.Person', on_delete=models.SET_NULL, null=True, related_name="advisor_programme")
    # signature = models.ImageField(upload_to='signature_img/%Y/%m/%d', null=True, blank=True, verbose_name="Class Teacher's Signature")
    created = models.DateTimeField(auto_now_add=True, null=True)
    updated = models.DateTimeField(auto_now=True, null=True)

    def __str__(self):
        try:
            return str(self.programme)
        except:
            return str(self.id)

    class Meta:
        ordering = ('-created',)
        # verbose_name = 'Class'
        # verbose_name_plural = 'Faculties'

class Spreadsheet(models.Model):
    session = models.ForeignKey('management.Session', on_delete=models.SET_NULL, null=True)
    semester = models.ForeignKey('management.Semester', on_delete=models.SET_NULL, null=True, blank=True)
    level = models.ForeignKey('management.Level', on_delete=models.SET_NULL, null=True, blank=True)
    programme = models.ForeignKey('management.Programme', on_delete=models.SET_NULL, null=True, blank=True)

    def __str__(self):
        try:
            return str(self.programme)
        except:
            return str(self.id)
    class Meta:
        ordering = ('-session',)

class Course(models.Model):
    course_code = models.CharField(max_length=11, null=True, verbose_name="Course Code")
    course_title = models.CharField(max_length=50, null=True, verbose_name="Course Title")
    credit_unit = models.PositiveIntegerField(default=0, verbose_name="Credit Unit")
    open = models.BooleanField(max_length=5, default = True)
    level = models.ForeignKey('management.Level', on_delete=models.SET_NULL, null=True)
    department = models.ForeignKey('management.Department', on_delete=models.SET_NULL, blank=True, null=True)
    programme = models.ManyToManyField('management.programme', blank=True, related_name="programmme_course")
    lecturer = models.ForeignKey('users.Person', on_delete=models.SET_NULL, null=True, verbose_name="Lecturer in Charge")

    # signature = models.ImageField(upload_to='signature_img/%Y/%m/%d', null=True, blank=True, verbose_name="Class Teacher's Signature")
    created = models.DateTimeField(auto_now_add=True, null=True)
    updated = models.DateTimeField(auto_now=True, null=True)

    def __str__(self):
        try:
            return str(self.course_code)
        except:
            return str(self.id)

    class Meta:
        ordering = ('course_code',)
        # verbose_name = 'Class'
        # verbose_name_plural = 'Faculties'

class CourseRegistration(models.Model):
    session = models.ForeignKey(Session, on_delete=models.SET_NULL, blank=True, null=True, related_name='session_course_registration')
    semester = models.ForeignKey(Semester, on_delete=models.SET_NULL, null=True, related_name='semester_course_registration')
    course = models.ForeignKey(Course, on_delete=models.SET_NULL, null=True, related_name='course_course_registration')
    undergrad = models.ForeignKey('users.Person', on_delete=models.SET_NULL, null=True, verbose_name="Student", related_name="student_course_registration")
    # recorded = models.BooleanField(max_length=5, default = False)
    static_level = models.CharField(max_length=3, blank=True, null=True)
    created = models.DateTimeField(auto_now_add=True, null=True)
    updated = models.DateTimeField(auto_now=True, null=True)

    def __str__(self):
        try:
            return str(self.course) + " | " + str(self.undergrad.username)
        except:
            return str(self.id)

    class Meta:
        ordering = ('-created',)
        verbose_name = 'Course Registration'
        verbose_name_plural = 'Course Registrations'

class Invoice(models.Model):
    undergrad = models.ForeignKey('users.Person', on_delete=models.SET_NULL, null=True)
    created = models.DateTimeField(auto_now_add=True, null=True)
    updated = models.DateTimeField(auto_now=True, null=True)

    def __str__(self):
        try:
            return str(self.undergrad)
        except:
            return str(self.id)

    class Meta:
        ordering = ('-created',)
